@@ -61,6 +61,14 @@
 		0xF1 ^ 0xFF, 0x77 ^ 0xFF, 0xBF ^ 0xFF, 0x17 ^ 0xFF,	\
 	}
 
+static const char *help_msg =
+#ifdef LTTNG_EMBED_HELP
+#include <lttng-crash.1.h>
+#else
+NULL
+#endif
+;
+
 /*
  * Non-static to ensure the compiler does not optimize away the xor.
  */
@@ -207,10 +215,10 @@ static struct option long_options[] = {
 
 static void usage(void)
 {
-	int ret = utils_show_man_page(1, "lttng-crash");
+	int ret = utils_show_help(1, "lttng-crash", help_msg);
 
 	if (ret) {
-		ERR("Cannot view man page lttng-crash(1)");
+		ERR("Cannot show --help for `lttng-crash`");
 		perror("exec");
 		exit(EXIT_FAILURE);
 	}
@@ -490,7 +498,8 @@ int check_magic(const uint8_t *magic)
 }
 
 static
-int get_crash_layout(struct lttng_crash_layout *layout, int fd)
+int get_crash_layout(struct lttng_crash_layout *layout, int fd,
+		const char *input_file)
 {
 	char *map;
 	int ret = 0, unmapret;
@@ -501,7 +510,21 @@ int get_crash_layout(struct lttng_crash_layout *layout, int fd)
 	const struct crash_abi_unknown *abi;
 	uint16_t endian;
 	enum lttng_crash_type layout_type;
+	struct stat stat;
 
+	ret = fstat(fd, &stat);
+	if (ret < 0) {
+		PERROR("Failed to fstat '%s'", input_file);
+		return -1;
+	}
+	if (stat.st_size < RB_CRASH_DUMP_ABI_LEN) {
+		ERR("File '%s' truncated: file length of %" PRIi64
+				" bytes does not meet the minimal expected "
+				"length of %d bytes",
+				input_file, (int64_t) stat.st_size,
+				RB_CRASH_DUMP_ABI_LEN);
+		return -1;
+	}
 	map = mmap(NULL, RB_CRASH_DUMP_ABI_LEN, PROT_READ, MAP_PRIVATE,
 		fd, 0);
 	if (map == MAP_FAILED) {
@@ -830,7 +853,7 @@ int extract_file(int output_dir_fd, const char *output_file,
 	}
 
 	/* Query the crash ABI layout */
-	ret = get_crash_layout(&layout, fd_src);
+	ret = get_crash_layout(&layout, fd_src, input_file);
 	if (ret) {
 		goto close_src;
 	}
