@@ -21,17 +21,26 @@
 #include <string.h>
 #include <unistd.h>
 
+#include <lttng/event.h>
+#include <lttng/lttng-error.h>
+#include <lttng/userspace-probe.h>
+#include <lttng/userspace-probe-internal.h>
+
 #include <common/common.h>
 #include <common/defaults.h>
+#include <common/trace-chunk.h>
+#include <common/macros.h>
 
 #include "consumer.h"
 #include "trace-kernel.h"
+#include "lttng-sessiond.h"
+#include "notification-thread-commands.h"
 
 /*
  * Find the channel name for the given kernel session.
  */
 struct ltt_kernel_channel *trace_kernel_get_channel_by_name(
-		char *name, struct ltt_kernel_session *session)
+		const char *name, struct ltt_kernel_session *session)
 {
 	struct ltt_kernel_channel *chan;
 
@@ -179,6 +188,7 @@ struct ltt_kernel_channel *trace_kernel_create_channel(
 		struct lttng_channel *chan)
 {
 	struct ltt_kernel_channel *lkc;
+	struct lttng_channel_extended *extended = NULL;
 
 	assert(chan);
 
@@ -191,10 +201,18 @@ struct ltt_kernel_channel *trace_kernel_create_channel(
 	lkc->channel = zmalloc(sizeof(struct lttng_channel));
 	if (lkc->channel == NULL) {
 		PERROR("lttng_channel zmalloc");
-		free(lkc);
+		goto error;
+	}
+
+	extended = zmalloc(sizeof(struct lttng_channel_extended));
+	if (!extended) {
+		PERROR("lttng_channel_channel zmalloc");
 		goto error;
 	}
 	memcpy(lkc->channel, chan, sizeof(struct lttng_channel));
+	memcpy(extended, chan->attr.extended.ptr, sizeof(struct lttng_channel_extended));
+	lkc->channel->attr.extended.ptr = extended;
+	extended = NULL;
 
 	/*
 	 * If we receive an empty string for channel name, it means the
@@ -210,6 +228,7 @@ struct ltt_kernel_channel *trace_kernel_create_channel(
 	lkc->stream_count = 0;
 	lkc->event_count = 0;
 	lkc->enabled = 1;
+	lkc->published_to_notification_thread = false;
 	/* Init linked list */
 	CDS_INIT_LIST_HEAD(&lkc->events_list.head);
 	CDS_INIT_LIST_HEAD(&lkc->stream_list.head);
@@ -218,6 +237,11 @@ struct ltt_kernel_channel *trace_kernel_create_channel(
 	return lkc;
 
 error:
+	if (lkc) {
+		free(lkc->channel);
+	}
+	free(extended);
+	free(lkc);
 	return NULL;
 }
 
@@ -275,18 +299,23 @@ error:
  *
  * Return pointer to structure or NULL.
  */
-struct ltt_kernel_event *trace_kernel_create_event(struct lttng_event *ev,
-		char *filter_expression, struct lttng_filter_bytecode *filter)
+enum lttng_error_code trace_kernel_create_event(
+		struct lttng_event *ev, char *filter_expression,
+		struct lttng_filter_bytecode *filter,
+		struct ltt_kernel_event **kernel_event)
 {
-	struct ltt_kernel_event *lke;
+	enum lttng_error_code ret;
 	struct lttng_kernel_event *attr;
+	struct ltt_kernel_event *local_kernel_event;
+	struct lttng_userspace_probe_location *userspace_probe_location = NULL;
 
 	assert(ev);
 
-	lke = zmalloc(sizeof(struct ltt_kernel_event));
+	local_kernel_event = zmalloc(sizeof(struct ltt_kernel_event));
 	attr = zmalloc(sizeof(struct lttng_kernel_event));
-	if (lke == NULL || attr == NULL) {
+	if (local_kernel_event == NULL || attr == NULL) {
 		PERROR("kernel event zmalloc");
+		ret = LTTNG_ERR_NOMEM;
 		goto error;
 	}
 
@@ -299,6 +328,84 @@ struct ltt_kernel_event *trace_kernel_create_event(struct lttng_event *ev,
 				ev->attr.probe.symbol_name, LTTNG_KERNEL_SYM_NAME_LEN);
 		attr->u.kprobe.symbol_name[LTTNG_KERNEL_SYM_NAME_LEN - 1] = '\0';
 		break;
+	case LTTNG_EVENT_USERSPACE_PROBE:
+	{
+		const struct lttng_userspace_probe_location* location = NULL;
+		const struct lttng_userspace_probe_location_lookup_method *lookup = NULL;
+
+		location = lttng_event_get_userspace_probe_location(ev);
+		if (!location) {
+			ret = LTTNG_ERR_PROBE_LOCATION_INVAL;
+			goto error;
+		}
+
+		/*
+		 * From this point on, the specific term 'uprobe' is used
+		 * instead of the generic 'userspace probe' because it's the
+		 * technology used at the moment for this instrumentation.
+		 * LTTng currently implements userspace probes using uprobes.
+		 * In the interactions with the kernel tracer, we use the
+		 * uprobe term.
+		 */
+		attr->instrumentation = LTTNG_KERNEL_UPROBE;
+
+		/*
+		 * Only the elf lookup method is supported at the moment.
+		 */
+		lookup = lttng_userspace_probe_location_get_lookup_method(
+				location);
+		if (!lookup) {
+			ret = LTTNG_ERR_PROBE_LOCATION_INVAL;
+			goto error;
+		}
+
+		/*
+		 * From the kernel tracer's perspective, all userspace probe
+		 * event types are all the same: a file and an offset.
+		 */
+		switch (lttng_userspace_probe_location_lookup_method_get_type(lookup)) {
+		case LTTNG_USERSPACE_PROBE_LOCATION_LOOKUP_METHOD_TYPE_FUNCTION_ELF:
+			/* Get the file descriptor on the target binary. */
+			attr->u.uprobe.fd =
+					lttng_userspace_probe_location_function_get_binary_fd(location);
+
+			/*
+			 * Save a reference to the probe location used during
+			 * the listing of events. Close its FD since it won't
+			 * be needed for listing.
+			 */
+			userspace_probe_location =
+					lttng_userspace_probe_location_copy(location);
+			ret = lttng_userspace_probe_location_function_set_binary_fd(
+					userspace_probe_location, -1);
+			if (ret) {
+				goto error;
+			}
+			break;
+		case LTTNG_USERSPACE_PROBE_LOCATION_LOOKUP_METHOD_TYPE_TRACEPOINT_SDT:
+			/* Get the file descriptor on the target binary. */
+			attr->u.uprobe.fd =
+					lttng_userspace_probe_location_tracepoint_get_binary_fd(location);
+
+			/*
+			 * Save a reference to the probe location used during the listing of
+			 * events. Close its FD since it won't be needed for listing.
+			 */
+			userspace_probe_location =
+					lttng_userspace_probe_location_copy(location);
+			ret = lttng_userspace_probe_location_tracepoint_set_binary_fd(
+					userspace_probe_location, -1);
+			if (ret) {
+				goto error;
+			}
+			break;
+		default:
+			DBG("Unsupported lookup method type");
+			ret = LTTNG_ERR_PROBE_LOCATION_INVAL;
+			goto error;
+		}
+		break;
+	}
 	case LTTNG_EVENT_FUNCTION:
 		attr->instrumentation = LTTNG_KERNEL_KRETPROBE;
 		attr->u.kretprobe.addr = ev->attr.probe.addr;
@@ -324,6 +431,7 @@ struct ltt_kernel_event *trace_kernel_create_event(struct lttng_event *ev,
 		break;
 	default:
 		ERR("Unknown kernel instrumentation type (%d)", ev->type);
+		ret = LTTNG_ERR_INVALID;
 		goto error;
 	}
 
@@ -332,20 +440,23 @@ struct ltt_kernel_event *trace_kernel_create_event(struct lttng_event *ev,
 	attr->name[LTTNG_KERNEL_SYM_NAME_LEN - 1] = '\0';
 
 	/* Setting up a kernel event */
-	lke->fd = -1;
-	lke->event = attr;
-	lke->enabled = 1;
-	lke->filter_expression = filter_expression;
-	lke->filter = filter;
+	local_kernel_event->fd = -1;
+	local_kernel_event->event = attr;
+	local_kernel_event->enabled = 1;
+	local_kernel_event->filter_expression = filter_expression;
+	local_kernel_event->filter = filter;
+	local_kernel_event->userspace_probe_location = userspace_probe_location;
 
-	return lke;
+	*kernel_event = local_kernel_event;
+
+	return LTTNG_OK;
 
 error:
 	free(filter_expression);
 	free(filter);
-	free(lke);
+	free(local_kernel_event);
 	free(attr);
-	return NULL;
+	return ret;
 }
 
 /*
@@ -355,6 +466,7 @@ error:
  */
 struct ltt_kernel_metadata *trace_kernel_create_metadata(void)
 {
+	int ret;
 	struct ltt_kernel_metadata *lkm;
 	struct lttng_channel *chan;
 
@@ -365,13 +477,38 @@ struct ltt_kernel_metadata *trace_kernel_create_metadata(void)
 		goto error;
 	}
 
+	ret = lttng_strncpy(
+			chan->name, DEFAULT_METADATA_NAME, sizeof(chan->name));
+	if (ret) {
+		ERR("Failed to initialize metadata channel name to `%s`",
+				DEFAULT_METADATA_NAME);
+		goto error;
+	}
+
 	/* Set default attributes */
-	chan->attr.overwrite = DEFAULT_CHANNEL_OVERWRITE;
+	chan->attr.overwrite = DEFAULT_METADATA_OVERWRITE;
 	chan->attr.subbuf_size = default_get_metadata_subbuf_size();
 	chan->attr.num_subbuf = DEFAULT_METADATA_SUBBUF_NUM;
-	chan->attr.switch_timer_interval = DEFAULT_KERNEL_CHANNEL_SWITCH_TIMER;
-	chan->attr.read_timer_interval = DEFAULT_KERNEL_CHANNEL_READ_TIMER;
-	chan->attr.output = DEFAULT_KERNEL_CHANNEL_OUTPUT;
+	chan->attr.switch_timer_interval = DEFAULT_METADATA_SWITCH_TIMER;
+	chan->attr.read_timer_interval = DEFAULT_METADATA_READ_TIMER;;
+
+
+	/*
+	 * The metadata channel of kernel sessions must use the "mmap"
+	 * back-end since the consumer daemon accumulates complete
+	 * metadata units before sending them to the relay daemon in
+	 * live mode. The consumer daemon also needs to extract the contents
+	 * of the metadata cache when computing a rotation position.
+	 *
+	 * In both cases, it is not possible to rely on the splice
+	 * back-end as the consumer daemon may need to accumulate more
+	 * content than can be backed by the ring buffer's underlying
+	 * pages.
+	 */
+	chan->attr.output = LTTNG_EVENT_MMAP;
+	chan->attr.tracefile_size = 0;
+	chan->attr.tracefile_count = 0;
+	chan->attr.live_timer_interval = 0;
 
 	/* Init metadata */
 	lkm->fd = -1;
@@ -499,6 +636,7 @@ void trace_kernel_destroy_channel(struct ltt_kernel_channel *channel)
 	struct ltt_kernel_event *event, *etmp;
 	struct ltt_kernel_context *ctx, *ctmp;
 	int ret;
+	enum lttng_error_code status;
 
 	assert(channel);
 
@@ -529,6 +667,14 @@ void trace_kernel_destroy_channel(struct ltt_kernel_channel *channel)
 	/* Remove from channel list */
 	cds_list_del(&channel->list);
 
+	if (notification_thread_handle
+			&& channel->published_to_notification_thread) {
+		status = notification_thread_command_remove_channel(
+				notification_thread_handle,
+				channel->key, LTTNG_DOMAIN_KERNEL);
+		assert(status == LTTNG_OK);
+	}
+	free(channel->channel->attr.extended.ptr);
 	free(channel->channel);
 	free(channel);
 }
@@ -591,7 +737,11 @@ void trace_kernel_destroy_session(struct ltt_kernel_session *session)
 	cds_list_for_each_entry_safe(channel, ctmp, &session->channel_list.head, list) {
 		trace_kernel_destroy_channel(channel);
 	}
+}
 
+/* Free elements needed by destroy notifiers. */
+void trace_kernel_free_session(struct ltt_kernel_session *session)
+{
 	/* Wipe consumer output object */
 	consumer_output_put(session->consumer);
 
