@@ -27,6 +27,8 @@
 #include "ust-app.h"
 #include "ust-field-utils.h"
 #include "utils.h"
+#include "lttng-sessiond.h"
+#include "notification-thread-commands.h"
 
 
 /*
@@ -85,10 +87,10 @@ no_match:
 	return 0;
 }
 
-static unsigned long ht_hash_event(void *_key, unsigned long seed)
+static unsigned long ht_hash_event(const void *_key, unsigned long seed)
 {
 	uint64_t hashed_key;
-	struct ust_registry_event *key = _key;
+	const struct ust_registry_event *key = _key;
 
 	assert(key);
 
@@ -719,13 +721,23 @@ void destroy_channel_rcu(struct rcu_head *head)
  * free the registry pointer since it might not have been allocated before so
  * it's the caller responsability.
  */
-static void destroy_channel(struct ust_registry_channel *chan)
+static void destroy_channel(struct ust_registry_channel *chan, bool notif)
 {
 	struct lttng_ht_iter iter;
 	struct ust_registry_event *event;
+	enum lttng_error_code cmd_ret;
 
 	assert(chan);
 
+	if (notif) {
+		cmd_ret = notification_thread_command_remove_channel(
+				notification_thread_handle, chan->consumer_key,
+				LTTNG_DOMAIN_UST);
+		if (cmd_ret != LTTNG_OK) {
+			ERR("Failed to remove channel from notification thread");
+		}
+	}
+
 	rcu_read_lock();
 	/* Destroy all event associated with this registry. */
 	cds_lfht_for_each_entry(chan->ht->ht, &iter.iter, event, node.node) {
@@ -783,7 +795,7 @@ int ust_registry_channel_add(struct ust_registry_session *session,
 	return 0;
 
 error:
-	destroy_channel(chan);
+	destroy_channel(chan, false);
 error_alloc:
 	return ret;
 }
@@ -822,7 +834,7 @@ end:
  * Remove channel using key from registry and free memory.
  */
 void ust_registry_channel_del_free(struct ust_registry_session *session,
-		uint64_t key)
+		uint64_t key, bool notif)
 {
 	struct lttng_ht_iter iter;
 	struct ust_registry_channel *chan;
@@ -841,7 +853,7 @@ void ust_registry_channel_del_free(struct ust_registry_session *session,
 	ret = lttng_ht_del(session->channels, &iter);
 	assert(!ret);
 	rcu_read_unlock();
-	destroy_channel(chan);
+	destroy_channel(chan, notif);
 
 end:
 	return;
@@ -868,7 +880,9 @@ int ust_registry_session_init(struct ust_registry_session **sessionp,
 		const char *root_shm_path,
 		const char *shm_path,
 		uid_t euid,
-		gid_t egid)
+		gid_t egid,
+		uint64_t tracing_id,
+		uid_t tracing_uid)
 {
 	int ret;
 	struct ust_registry_session *session;
@@ -932,7 +946,7 @@ int ust_registry_session_init(struct ust_registry_session **sessionp,
 
 	session->enums = lttng_ht_new(0, LTTNG_HT_TYPE_STRING);
 	if (!session->enums) {
-		ret = -ENOMEM;
+		ERR("Failed to create enums hash table");
 		goto error;
 	}
 	/* hash/match functions are specified at call site. */
@@ -950,6 +964,9 @@ int ust_registry_session_init(struct ust_registry_session **sessionp,
 		goto error;
 	}
 
+	session->tracing_id = tracing_id;
+	session->tracing_uid = tracing_uid;
+
 	pthread_mutex_lock(&session->lock);
 	ret = ust_metadata_session_statedump(session, app, major, minor);
 	pthread_mutex_unlock(&session->lock);
@@ -996,7 +1013,7 @@ void ust_registry_session_destroy(struct ust_registry_session *reg)
 			/* Delete the node from the ht and free it. */
 			ret = lttng_ht_del(reg->channels, &iter);
 			assert(!ret);
-			destroy_channel(chan);
+			destroy_channel(chan, true);
 		}
 		rcu_read_unlock();
 		ht_cleanup_push(reg->channels);
@@ -1018,8 +1035,9 @@ void ust_registry_session_destroy(struct ust_registry_session *reg)
 		/*
 		 * Try deleting the directory hierarchy.
 		 */
-		(void) run_as_recursive_rmdir(reg->root_shm_path,
-				reg->uid, reg->gid);
+		(void) run_as_rmdir_recursive(reg->root_shm_path,
+				reg->uid, reg->gid,
+				LTTNG_DIRECTORY_HANDLE_SKIP_NON_EMPTY_FLAG);
 	}
 	/* Destroy the enum hash table */
 	if (reg->enums) {
