@@ -24,6 +24,7 @@
 #include <sys/types.h>
 #include <unistd.h>
 #include <stdbool.h>
+#include <lttng/lttng.h>
 
 #include "../command.h"
 
@@ -35,6 +36,12 @@ static char *opt_session_name;
 static int opt_destroy_all;
 static int opt_no_wait;
 
+#ifdef LTTNG_EMBED_HELP
+static const char help_msg[] =
+#include <lttng-destroy.1.h>
+;
+#endif
+
 /* Mi writer */
 static struct mi_writer *writer;
 
@@ -62,16 +69,21 @@ static int destroy_session(struct lttng_session *session)
 {
 	int ret;
 	char *session_name = NULL;
-	bool session_was_stopped;
+	bool session_was_already_stopped;
+	enum lttng_error_code ret_code;
+	struct lttng_destruction_handle *handle = NULL;
+	enum lttng_destruction_handle_status status;
+	bool newline_needed = false, printed_destroy_msg = false;
+	enum lttng_rotation_state rotation_state;
+	char *stats_str = NULL;
 
 	ret = lttng_stop_tracing_no_wait(session->name);
 	if (ret < 0 && ret != -LTTNG_ERR_TRACE_ALREADY_STOPPED) {
 		ERR("%s", lttng_strerror(ret));
 	}
-	session_was_stopped = ret == -LTTNG_ERR_TRACE_ALREADY_STOPPED;
-	if (!opt_no_wait) {
-		bool printed_wait_msg = false;
 
+	session_was_already_stopped = ret == -LTTNG_ERR_TRACE_ALREADY_STOPPED;
+	if (!opt_no_wait) {
 		do {
 			ret = lttng_data_pending(session->name);
 			if (ret < 0) {
@@ -80,39 +92,130 @@ static int destroy_session(struct lttng_session *session)
 			}
 
 			/*
-			 * Data sleep time before retrying (in usec). Don't sleep if the call
-			 * returned value indicates availability.
+			 * Data sleep time before retrying (in usec). Don't
+			 * sleep if the call returned value indicates
+			 * availability.
 			 */
 			if (ret) {
-				if (!printed_wait_msg) {
-					_MSG("Waiting for data availability");
+				if (!printed_destroy_msg) {
+					_MSG("Destroying session %s",
+							session->name);
+					newline_needed = true;
+					printed_destroy_msg = true;
 					fflush(stdout);
 				}
 
-				printed_wait_msg = true;
-				usleep(DEFAULT_DATA_AVAILABILITY_WAIT_TIME);
+				usleep(DEFAULT_DATA_AVAILABILITY_WAIT_TIME_US);
 				_MSG(".");
 				fflush(stdout);
 			}
 		} while (ret != 0);
-		if (printed_wait_msg) {
-			MSG("");
-		}
 	}
-	if (!session_was_stopped) {
+
+	if (!session_was_already_stopped) {
 		/*
 		 * Don't print the event and packet loss warnings since the user
 		 * already saw them when stopping the trace.
 		 */
-		print_session_stats(session->name);
+		ret = get_session_stats_str(session->name, &stats_str);
+		if (ret < 0) {
+			goto error;
+		}
 	}
 
-	ret = lttng_destroy_session_no_wait(session->name);
-	if (ret < 0) {
+	ret_code = lttng_destroy_session_ext(session->name, &handle);
+	if (ret_code != LTTNG_OK) {
+		ret = -ret_code;
 		goto error;
 	}
 
-	MSG("Session %s destroyed", session->name);
+	if (opt_no_wait) {
+		goto skip_wait_rotation;
+	}
+
+	do {
+		status = lttng_destruction_handle_wait_for_completion(
+				handle, DEFAULT_DATA_AVAILABILITY_WAIT_TIME_US /
+							USEC_PER_MSEC);
+		switch (status) {
+		case LTTNG_DESTRUCTION_HANDLE_STATUS_TIMEOUT:
+			if (!printed_destroy_msg) {
+				_MSG("Destroying session %s", session->name);
+				newline_needed = true;
+				printed_destroy_msg = true;
+			}
+			_MSG(".");
+			fflush(stdout);
+			break;
+		case LTTNG_DESTRUCTION_HANDLE_STATUS_COMPLETED:
+			break;
+		default:
+			ERR("%sFailed to wait for the completion of the destruction of session \"%s\"",
+					newline_needed ? "\n" : "",
+					session->name);
+			newline_needed = false;
+			ret = -1;
+			goto error;
+		}
+	} while (status == LTTNG_DESTRUCTION_HANDLE_STATUS_TIMEOUT);
+
+	status = lttng_destruction_handle_get_result(handle, &ret_code);
+	if (status != LTTNG_DESTRUCTION_HANDLE_STATUS_OK) {
+		ERR("%sFailed to get the result of session destruction",
+				newline_needed ? "\n" : "");
+		ret = -1;
+		newline_needed = false;
+		goto error;
+	}
+	if (ret_code != LTTNG_OK) {
+		ret = -ret_code;
+		goto error;
+	}
+
+	status = lttng_destruction_handle_get_rotation_state(
+			handle, &rotation_state);
+	if (status != LTTNG_DESTRUCTION_HANDLE_STATUS_OK) {
+		ERR("%sFailed to get rotation state from destruction handle",
+				newline_needed ? "\n" : "");
+		newline_needed = false;
+		goto skip_wait_rotation;
+	}
+
+	switch (rotation_state) {
+	case LTTNG_ROTATION_STATE_NO_ROTATION:
+		break;
+	case LTTNG_ROTATION_STATE_COMPLETED:
+	{
+		const struct lttng_trace_archive_location *location;
+
+		status = lttng_destruction_handle_get_archive_location(
+				handle, &location);
+		if (status == LTTNG_DESTRUCTION_HANDLE_STATUS_OK) {
+			ret = print_trace_archive_location(
+					location, session->name);
+			if (ret) {
+				ERR("%sFailed to print the location of trace archive",
+						newline_needed ? "\n" : "");
+				newline_needed = false;
+				goto skip_wait_rotation;
+			}
+			break;
+		}
+		/* fall-through. */
+	}
+	default:
+		ERR("%sFailed to get the location of the rotation performed during the session's destruction",
+				newline_needed ? "\n" : "");
+		newline_needed = false;
+		goto skip_wait_rotation;
+	}
+skip_wait_rotation:
+	MSG("%sSession %s destroyed", newline_needed ? "\n" : "",
+			session->name);
+	newline_needed = false;
+	if (stats_str) {
+		MSG("%s", stats_str);
+	}
 
 	session_name = get_session_name_quiet();
 	if (session_name && !strncmp(session->name, session_name, NAME_MAX)) {
@@ -129,7 +232,12 @@ static int destroy_session(struct lttng_session *session)
 
 	ret = CMD_SUCCESS;
 error:
+	if (newline_needed) {
+		MSG("");
+	}
+	lttng_destruction_handle_destroy(handle);
 	free(session_name);
+	free(stats_str);
 	return ret;
 }
 
@@ -140,23 +248,27 @@ error:
  */
 static int destroy_all_sessions(struct lttng_session *sessions, int count)
 {
-	int i, ret = CMD_SUCCESS;
+	int i;
+	bool error_occurred = false;
 
+	assert(count >= 0);
 	if (count == 0) {
 		MSG("No session found, nothing to do.");
-	} else if (count < 0) {
-		ERR("%s", lttng_strerror(ret));
-		goto error;
 	}
 
 	for (i = 0; i < count; i++) {
-		ret = destroy_session(&sessions[i]);
+		int ret = destroy_session(&sessions[i]);
+
 		if (ret < 0) {
-			goto error;
+			ERR("%s during the destruction of session \"%s\"",
+					lttng_strerror(ret),
+					sessions[i].name);
+			/* Continue to next session. */
+			error_occurred = true;
 		}
 	}
-error:
-	return ret;
+
+	return error_occurred ? CMD_ERROR : CMD_SUCCESS;
 }
 
 /*
@@ -262,8 +374,10 @@ int cmd_destroy(int argc, const char **argv)
 				command_ret = destroy_session(&sessions[i]);
 				if (command_ret) {
 					success = 0;
+					ERR("%s during the destruction of session \"%s\"",
+							lttng_strerror(command_ret),
+							sessions[i].name);
 				}
-
 			}
 		}
 
