@@ -40,6 +40,12 @@ static int opt_log4j;
 static int opt_python;
 static int opt_event_type;
 
+#ifdef LTTNG_EMBED_HELP
+static const char help_msg[] =
+#include <lttng-disable-event.1.h>
+;
+#endif
+
 enum {
 	OPT_HELP = 1,
 	OPT_TYPE_SYSCALL,
@@ -372,7 +378,9 @@ int cmd_disable_events(int argc, const char **argv)
 	}
 
 	ret = print_missing_or_multiple_domains(
-		opt_kernel + opt_userspace + opt_jul + opt_log4j + opt_python);
+			opt_kernel + opt_userspace + opt_jul + opt_log4j +
+					opt_python,
+			true);
 	if (ret) {
 		ret = CMD_ERROR;
 		goto end;
