@@ -24,6 +24,7 @@
 #include <assert.h>
 
 #include <common/mi-lttng.h>
+#include <common/time.h>
 #include <lttng/constant.h>
 
 #include "../command.h"
@@ -42,6 +43,12 @@ const char *indent4 = "    ";
 const char *indent6 = "      ";
 const char *indent8 = "        ";
 
+#ifdef LTTNG_EMBED_HELP
+static const char help_msg[] =
+#include <lttng-list.1.h>
+;
+#endif
+
 enum {
 	OPT_HELP = 1,
 	OPT_USERSPACE,
@@ -51,6 +58,9 @@ enum {
 static struct lttng_handle *handle;
 static struct mi_writer *writer;
 
+/* Only set when listing a single session. */
+static struct lttng_session listed_session;
+
 static struct poptOption long_options[] = {
 	/* longName, shortName, argInfo, argPtr, value, descrip, argDesc */
 	{"help",	'h', POPT_ARG_NONE, 0, OPT_HELP, 0, 0},
@@ -240,6 +250,86 @@ end:
 	return exclusion_msg;
 }
 
+static void print_userspace_probe_location(struct lttng_event *event)
+{
+	const struct lttng_userspace_probe_location *location;
+	const struct lttng_userspace_probe_location_lookup_method *lookup_method;
+	enum lttng_userspace_probe_location_lookup_method_type lookup_type;
+
+	location = lttng_event_get_userspace_probe_location(event);
+	if (!location) {
+		MSG("Event has no userspace probe location");
+		return;
+	}
+
+	lookup_method = lttng_userspace_probe_location_get_lookup_method(location);
+	if (!lookup_method) {
+		MSG("Event has no userspace probe location lookup method");
+		return;
+	}
+
+	MSG("%s%s (type: userspace-probe)%s", indent6, event->name, enabled_string(event->enabled));
+
+	lookup_type = lttng_userspace_probe_location_lookup_method_get_type(lookup_method);
+
+	switch (lttng_userspace_probe_location_get_type(location)) {
+	case LTTNG_USERSPACE_PROBE_LOCATION_TYPE_UNKNOWN:
+		MSG("%sType: Unknown", indent8);
+		break;
+	case LTTNG_USERSPACE_PROBE_LOCATION_TYPE_FUNCTION:
+	{
+		const char *function_name;
+		char *binary_path;
+
+		MSG("%sType: Function", indent8);
+		function_name = lttng_userspace_probe_location_function_get_function_name(location);
+		binary_path = realpath(lttng_userspace_probe_location_function_get_binary_path(location), NULL);
+
+		MSG("%sBinary path:   %s", indent8, binary_path ? binary_path : "NULL");
+		MSG("%sFunction:      %s()", indent8, function_name ? function_name : "NULL");
+		switch (lookup_type) {
+		case LTTNG_USERSPACE_PROBE_LOCATION_LOOKUP_METHOD_TYPE_FUNCTION_ELF:
+			MSG("%sLookup method: ELF", indent8);
+			break;
+		case LTTNG_USERSPACE_PROBE_LOCATION_LOOKUP_METHOD_TYPE_FUNCTION_DEFAULT:
+			MSG("%sLookup method: default", indent8);
+			break;
+		default:
+			MSG("%sLookup method: INVALID LOOKUP TYPE ENCOUNTERED", indent8);
+			break;
+		}
+
+		free(binary_path);
+		break;
+	}
+	case LTTNG_USERSPACE_PROBE_LOCATION_TYPE_TRACEPOINT:
+	{
+		const char *probe_name, *provider_name;
+		char *binary_path;
+
+		MSG("%sType: Tracepoint", indent8);
+		probe_name = lttng_userspace_probe_location_tracepoint_get_probe_name(location);
+		provider_name = lttng_userspace_probe_location_tracepoint_get_provider_name(location);
+		binary_path = realpath(lttng_userspace_probe_location_tracepoint_get_binary_path(location), NULL);
+		MSG("%sBinary path:   %s", indent8, binary_path ? binary_path : "NULL");
+		MSG("%sTracepoint:    %s:%s", indent8, provider_name ? provider_name : "NULL", probe_name ? probe_name : "NULL");
+		switch (lookup_type) {
+		case LTTNG_USERSPACE_PROBE_LOCATION_LOOKUP_METHOD_TYPE_TRACEPOINT_SDT:
+			MSG("%sLookup method: SDT", indent8);
+			break;
+		default:
+			MSG("%sLookup method: INVALID LOOKUP TYPE ENCOUNTERED", indent8);
+			break;
+		}
+
+		free(binary_path);
+		break;
+	}
+	default:
+		ERR("Invalid probe type encountered");
+	}
+}
+
 /*
  * Pretty print single event.
  */
@@ -315,6 +405,9 @@ static void print_events(struct lttng_event *event)
 			MSG("%ssymbol: %s", indent8, event->attr.probe.symbol_name);
 		}
 		break;
+	case LTTNG_EVENT_USERSPACE_PROBE:
+		print_userspace_probe_location(event);
+		break;
 	case LTTNG_EVENT_FUNCTION_ENTRY:
 		MSG("%s%s (type: function)%s%s", indent6,
 				event->name, enabled_string(event->enabled),
@@ -334,6 +427,8 @@ static void print_events(struct lttng_event *event)
 				safe_string(filter_msg));
 		break;
 	case LTTNG_EVENT_ALL:
+		/* Fall-through. */
+	default:
 		/* We should never have "all" events in list. */
 		assert(0);
 		break;
@@ -1142,7 +1237,7 @@ static int list_events(const char *channel_name)
 		}
 	} else {
 		/* Pretty print */
-		MSG("\n%sEvents:", indent4);
+		MSG("\n%sEvent rules:", indent4);
 		if (count == 0) {
 			MSG("%sNone\n", indent6);
 			goto end;
@@ -1160,13 +1255,31 @@ error:
 	return ret;
 }
 
+static
+void print_timer(const char *timer_name, uint32_t space_count, int64_t value)
+{
+	uint32_t i;
+
+	_MSG("%s%s:", indent6, timer_name);
+	for (i = 0; i < space_count; i++) {
+		_MSG(" ");
+	}
+
+	if (value) {
+		MSG("%" PRId64 " %s", value, USEC_UNIT);
+	} else {
+		MSG("inactive");
+	}
+}
+
 /*
  * Pretty print channel
  */
 static void print_channel(struct lttng_channel *channel)
 {
 	int ret;
-	uint64_t discarded_events, lost_packets;
+	uint64_t discarded_events, lost_packets, monitor_timer_interval;
+	int64_t blocking_timeout;
 
 	ret = lttng_channel_get_discarded_event_count(channel,
 			&discarded_events);
@@ -1182,26 +1295,83 @@ static void print_channel(struct lttng_channel *channel)
 		return;
 	}
 
-	MSG("- %s:%s\n", channel->name, enabled_string(channel->enabled));
+	ret = lttng_channel_get_monitor_timer_interval(channel,
+			&monitor_timer_interval);
+	if (ret) {
+		ERR("Failed to retrieve monitor interval of channel");
+		return;
+	}
 
+	ret = lttng_channel_get_blocking_timeout(channel,
+			&blocking_timeout);
+	if (ret) {
+		ERR("Failed to retrieve blocking timeout of channel");
+		return;
+	}
+
+	MSG("- %s:%s\n", channel->name, enabled_string(channel->enabled));
 	MSG("%sAttributes:", indent4);
-	MSG("%soverwrite mode: %d", indent6, channel->attr.overwrite);
-	MSG("%ssubbuffers size: %" PRIu64, indent6, channel->attr.subbuf_size);
-	MSG("%snumber of subbuffers: %" PRIu64, indent6, channel->attr.num_subbuf);
-	MSG("%sswitch timer interval: %u", indent6, channel->attr.switch_timer_interval);
-	MSG("%sread timer interval: %u", indent6, channel->attr.read_timer_interval);
-	MSG("%strace file count: %" PRIu64, indent6, channel->attr.tracefile_count);
-	MSG("%strace file size (bytes): %" PRIu64, indent6, channel->attr.tracefile_size);
-	MSG("%sdiscarded events: %" PRIu64, indent6, discarded_events);
-	MSG("%slost packets: %" PRIu64, indent6, lost_packets);
+	MSG("%sEvent-loss mode:  %s", indent6, channel->attr.overwrite ? "overwrite" : "discard");
+	MSG("%sSub-buffer size:  %" PRIu64 " bytes", indent6, channel->attr.subbuf_size);
+	MSG("%sSub-buffer count: %" PRIu64, indent6, channel->attr.num_subbuf);
+
+	print_timer("Switch timer", 5, channel->attr.switch_timer_interval);
+	print_timer("Read timer",  7, channel->attr.read_timer_interval);
+	print_timer("Monitor timer", 4, monitor_timer_interval);
+
+	if (!channel->attr.overwrite) {
+		if (blocking_timeout == -1) {
+			MSG("%sBlocking timeout: infinite", indent6);
+		} else {
+			MSG("%sBlocking timeout: %" PRId64 " %s", indent6,
+					blocking_timeout, USEC_UNIT);
+		}
+	}
+
+	MSG("%sTrace file count: %" PRIu64 " per stream", indent6,
+			channel->attr.tracefile_count == 0 ?
+				1 : channel->attr.tracefile_count);
+	if (channel->attr.tracefile_size != 0 ) {
+		MSG("%sTrace file size:  %" PRIu64 " bytes", indent6,
+				channel->attr.tracefile_size);
+	} else {
+		MSG("%sTrace file size:  %s", indent6, "unlimited");
+	}
 	switch (channel->attr.output) {
 		case LTTNG_EVENT_SPLICE:
-			MSG("%soutput: splice()", indent6);
+			MSG("%sOutput mode:      splice", indent6);
 			break;
 		case LTTNG_EVENT_MMAP:
-			MSG("%soutput: mmap()", indent6);
+			MSG("%sOutput mode:      mmap", indent6);
 			break;
 	}
+
+	MSG("\n%sStatistics:", indent4);
+	if (listed_session.snapshot_mode) {
+		/*
+		 * The lost packet count is omitted for sessions in snapshot
+		 * mode as it is misleading: it would indicate the number of
+		 * packets that the consumer could not extract during the
+		 * course of recording the snapshot. It does not have the
+		 * same meaning as the "regular" lost packet count that
+		 * would result from the consumer not keeping up with
+		 * event production in an overwrite-mode channel.
+		 *
+		 * A more interesting statistic would be the number of
+		 * packets lost between the first and last extracted
+		 * packets of a given snapshot (which prevents most analyses).
+		 */
+		MSG("%sNone", indent6);
+		goto skip_stats_printing;
+	}
+
+	if (!channel->attr.overwrite) {
+		MSG("%sDiscarded events: %" PRIu64, indent6, discarded_events);
+	} else {
+		MSG("%sLost packets:     %" PRIu64, indent6, lost_packets);
+	}
+skip_stats_printing:
+	return;
 }
 
 /*
@@ -1286,7 +1456,6 @@ static int list_channels(const char *channel_name)
 				count = 0;
 			} else {
 				ret = CMD_SUCCESS;
-				WARN("No kernel channel");
 				goto error_channels;
 			}
 			break;
@@ -1437,6 +1606,147 @@ end:
 	return ret;
 }
 
+static enum cmd_error_code print_periodic_rotation_schedule(
+		const struct lttng_rotation_schedule *schedule)
+{
+	enum cmd_error_code ret;
+	enum lttng_rotation_status status;
+	uint64_t value;
+
+	status = lttng_rotation_schedule_periodic_get_period(schedule,
+			&value);
+	if (status != LTTNG_ROTATION_STATUS_OK) {
+		ERR("Failed to retrieve period parameter from periodic rotation schedule.");
+		ret = CMD_ERROR;
+		goto end;
+	}
+
+	MSG("    timer period: %" PRIu64" %s", value, USEC_UNIT);
+	ret = CMD_SUCCESS;
+end:
+	return ret;
+}
+
+static enum cmd_error_code print_size_threshold_rotation_schedule(
+		const struct lttng_rotation_schedule *schedule)
+{
+	enum cmd_error_code ret;
+	enum lttng_rotation_status status;
+	uint64_t value;
+
+	status = lttng_rotation_schedule_size_threshold_get_threshold(schedule,
+			&value);
+	if (status != LTTNG_ROTATION_STATUS_OK) {
+		ERR("Failed to retrieve size parameter from size-based rotation schedule.");
+		ret = CMD_ERROR;
+		goto end;
+	}
+
+	MSG("    size threshold: %" PRIu64" bytes", value);
+	ret = CMD_SUCCESS;
+end:
+	return ret;
+}
+
+static enum cmd_error_code print_rotation_schedule(
+		const struct lttng_rotation_schedule *schedule)
+{
+	enum cmd_error_code ret;
+
+	switch (lttng_rotation_schedule_get_type(schedule)) {
+	case LTTNG_ROTATION_SCHEDULE_TYPE_SIZE_THRESHOLD:
+		ret = print_size_threshold_rotation_schedule(schedule);
+		break;
+	case LTTNG_ROTATION_SCHEDULE_TYPE_PERIODIC:
+		ret = print_periodic_rotation_schedule(schedule);
+		break;
+	default:
+		ret = CMD_ERROR;
+	}
+	return ret;
+}
+
+/*
+ * List the automatic rotation settings.
+ */
+static enum cmd_error_code list_rotate_settings(const char *session_name)
+{
+	int ret;
+	enum cmd_error_code cmd_ret = CMD_SUCCESS;
+	unsigned int count, i;
+	struct lttng_rotation_schedules *schedules = NULL;
+	enum lttng_rotation_status status;
+
+	ret = lttng_session_list_rotation_schedules(session_name, &schedules);
+	if (ret != LTTNG_OK) {
+		ERR("Failed to list session rotation schedules: %s", lttng_strerror(ret));
+		cmd_ret = CMD_ERROR;
+		goto end;
+	}
+
+	status = lttng_rotation_schedules_get_count(schedules, &count);
+	if (status != LTTNG_ROTATION_STATUS_OK) {
+		ERR("Failed to retrieve the number of session rotation schedules.");
+		cmd_ret = CMD_ERROR;
+		goto end;
+	}
+
+	if (count == 0) {
+		cmd_ret = CMD_SUCCESS;
+		goto end;
+	}
+
+	MSG("Automatic rotation schedules:");
+	if (lttng_opt_mi) {
+		ret = mi_lttng_writer_open_element(writer,
+				mi_lttng_element_rotation_schedules);
+		if (ret) {
+			cmd_ret = CMD_ERROR;
+			goto end;
+		}
+	}
+
+	for (i = 0; i < count; i++) {
+		enum cmd_error_code tmp_ret = CMD_SUCCESS;
+		const struct lttng_rotation_schedule *schedule;
+
+		schedule = lttng_rotation_schedules_get_at_index(schedules, i);
+		if (!schedule) {
+			ERR("Failed to retrieve session rotation schedule.");
+			cmd_ret = CMD_ERROR;
+			goto end;
+		}
+
+		if (lttng_opt_mi) {
+			ret = mi_lttng_rotation_schedule(writer, schedule);
+			if (ret) {
+				tmp_ret = CMD_ERROR;
+			}
+		} else {
+			tmp_ret = print_rotation_schedule(schedule);
+		}
+
+		/*
+		 * Report an error if the serialization of any of the
+		 * descriptors failed.
+		 */
+		cmd_ret = cmd_ret ? cmd_ret : tmp_ret;
+	}
+
+	_MSG("\n");
+	if (lttng_opt_mi) {
+		/* Close the rotation_schedules element. */
+		ret = mi_lttng_writer_close_element(writer);
+		if (ret) {
+			cmd_ret = CMD_ERROR;
+			goto end;
+		}
+	}
+end:
+	lttng_rotation_schedules_destroy(schedules);
+	return cmd_ret;
+}
+
 /*
  * Machine interface
  * Find the session with session_name as name
@@ -1518,7 +1828,7 @@ static int list_sessions(const char *session_name)
 	int ret = CMD_SUCCESS;
 	int count, i;
 	unsigned int session_found = 0;
-	struct lttng_session *sessions;
+	struct lttng_session *sessions = NULL;
 
 	count = lttng_list_sessions(&sessions);
 	DBG("Session count %d", count);
@@ -1531,7 +1841,7 @@ static int list_sessions(const char *session_name)
 	if (lttng_opt_mi) {
 		/* Mi */
 		if (session_name == NULL) {
-			/* List all session */
+			/* List all sessions */
 			ret = mi_list_sessions(sessions, count);
 		} else {
 			/* Note : this return an open session element */
@@ -1539,7 +1849,7 @@ static int list_sessions(const char *session_name)
 		}
 		if (ret) {
 			ret = CMD_ERROR;
-			goto error;
+			goto end;
 		}
 	} else {
 		/* Pretty print */
@@ -1552,7 +1862,6 @@ static int list_sessions(const char *session_name)
 			MSG("Available tracing sessions:");
 		}
 
-
 		for (i = 0; i < count; i++) {
 			if (session_name != NULL) {
 				if (strncmp(sessions[i].name, session_name, NAME_MAX) == 0) {
@@ -1560,18 +1869,25 @@ static int list_sessions(const char *session_name)
 					MSG("Tracing session %s: [%s%s]", session_name,
 							active_string(sessions[i].enabled),
 							snapshot_string(sessions[i].snapshot_mode));
-					MSG("%sTrace path: %s\n", indent4, sessions[i].path);
+					if (*sessions[i].path) {
+						MSG("%sTrace output: %s\n", indent4, sessions[i].path);
+					}
+					memcpy(&listed_session, &sessions[i],
+							sizeof(listed_session));
 					break;
 				}
 			} else {
-				MSG("  %d) %s (%s) [%s%s]", i + 1,
-						sessions[i].name, sessions[i].path,
+				MSG("  %d) %s [%s%s]", i + 1,
+						sessions[i].name,
 						active_string(sessions[i].enabled),
 						snapshot_string(sessions[i].snapshot_mode));
-				MSG("%sTrace path: %s", indent4, sessions[i].path);
+				if (*sessions[i].path) {
+					MSG("%sTrace output: %s", indent4, sessions[i].path);
+				}
 				if (sessions[i].live_timer_interval != 0) {
-					MSG("%sLive timer interval (usec): %u", indent4,
-							sessions[i].live_timer_interval);
+					MSG("%sLive timer interval: %u %s", indent4,
+							sessions[i].live_timer_interval,
+							USEC_UNIT);
 				}
 				MSG("");
 			}
@@ -1580,7 +1896,7 @@ static int list_sessions(const char *session_name)
 		if (!session_found && session_name != NULL) {
 			ERR("Session '%s' not found", session_name);
 			ret = CMD_ERROR;
-			goto error;
+			goto end;
 		}
 
 		if (session_name == NULL) {
@@ -1588,9 +1904,8 @@ static int list_sessions(const char *session_name)
 		}
 	}
 
-error:
-	free(sessions);
 end:
+	free(sessions);
 	return ret;
 }
 
@@ -1840,6 +2155,11 @@ int cmd_list(int argc, const char **argv)
 			goto end;
 		}
 
+		ret = list_rotate_settings(session_name);
+		if (ret) {
+			goto end;
+		}
+
 		/* Domain listing */
 		if (opt_domain) {
 			ret = list_domains(session_name);
