@@ -15,7 +15,7 @@
  * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-#define _GNU_SOURCE
+#define _LGPL_SOURCE
 #include <assert.h>
 #include <fcntl.h>
 #include <limits.h>
@@ -23,7 +23,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
-#include <config.h>
+#include <stdbool.h>
 
 #include <common/error.h>
 #include <common/defaults.h>
@@ -32,7 +32,17 @@
 
 #include "poll.h"
 
-unsigned int poll_max_size;
+/*
+ * Maximum number of fd we can monitor.
+ *
+ * For epoll(7), /proc/sys/fs/epoll/max_user_watches (since Linux 2.6.28) will
+ * be used for the maximum size of the poll set. If this interface is not
+ * available, according to the manpage, the max_user_watches value is 1/25 (4%)
+ * of the available low memory divided by the registration cost in bytes which
+ * is 90 bytes on a 32-bit kernel and 160 bytes on a 64-bit kernel.
+ *
+ */
+static unsigned int poll_max_size;
 
 /*
  * Resize the epoll events structure of the new size.
@@ -68,6 +78,7 @@ error:
 /*
  * Create epoll set and allocate returned events structure.
  */
+LTTNG_HIDDEN
 int compat_epoll_create(struct lttng_poll_event *events, int size, int flags)
 {
 	int ret;
@@ -77,8 +88,9 @@ int compat_epoll_create(struct lttng_poll_event *events, int size, int flags)
 	}
 
 	if (!poll_max_size) {
-		ERR("poll_max_size not initialized yet");
-		goto error;
+		if (lttng_poll_set_max_size()) {
+			goto error;
+		}
 	}
 
 	/* Don't bust the limit here */
@@ -119,6 +131,7 @@ error:
 /*
  * Add a fd to the epoll set with requesting events.
  */
+LTTNG_HIDDEN
 int compat_epoll_add(struct lttng_poll_event *events, int fd, uint32_t req_events)
 {
 	int ret;
@@ -166,6 +179,7 @@ error:
 /*
  * Remove a fd from the epoll set.
  */
+LTTNG_HIDDEN
 int compat_epoll_del(struct lttng_poll_event *events, int fd)
 {
 	int ret;
@@ -197,10 +211,54 @@ error:
 	return -1;
 }
 
+/*
+ * Set an fd's events.
+ */
+LTTNG_HIDDEN
+int compat_epoll_mod(struct lttng_poll_event *events, int fd, uint32_t req_events)
+{
+	int ret;
+	struct epoll_event ev;
+
+	if (events == NULL || fd < 0 || events->nb_fd == 0) {
+		goto error;
+	}
+
+	/*
+	 * Zero struct epoll_event to ensure all representations of its
+	 * union are zeroed.
+	 */
+	memset(&ev, 0, sizeof(ev));
+	ev.events = req_events;
+	ev.data.fd = fd;
+
+	ret = epoll_ctl(events->epfd, EPOLL_CTL_MOD, fd, &ev);
+	if (ret < 0) {
+		switch (errno) {
+		case ENOENT:
+		case EPERM:
+			/* Print PERROR and goto end not failing. Show must go on. */
+			PERROR("epoll_ctl MOD");
+			goto end;
+		default:
+			PERROR("epoll_ctl MOD fatal");
+			goto error;
+		}
+	}
+
+end:
+	return 0;
+
+error:
+	return -1;
+}
+
 /*
  * Wait on epoll set. This is a blocking call of timeout value.
  */
-int compat_epoll_wait(struct lttng_poll_event *events, int timeout)
+LTTNG_HIDDEN
+int compat_epoll_wait(struct lttng_poll_event *events, int timeout,
+		bool interruptible)
 {
 	int ret;
 	uint32_t new_size;
@@ -209,7 +267,6 @@ int compat_epoll_wait(struct lttng_poll_event *events, int timeout)
 		ERR("Wrong arguments in compat_epoll_wait");
 		goto error;
 	}
-	assert(events->nb_fd >= 0);
 
 	if (events->nb_fd == 0) {
 		errno = EINVAL;
@@ -233,10 +290,11 @@ int compat_epoll_wait(struct lttng_poll_event *events, int timeout)
 
 	do {
 		ret = epoll_wait(events->epfd, events->events, events->nb_fd, timeout);
-	} while (ret == -1 && errno == EINTR);
+	} while (!interruptible && ret == -1 && errno == EINTR);
 	if (ret < 0) {
-		/* At this point, every error is fatal */
-		PERROR("epoll_wait");
+		if (errno != EINTR) {
+			PERROR("epoll_wait");
+		}
 		goto error;
 	}
 
@@ -253,6 +311,7 @@ error:
 /*
  * Setup poll set maximum size.
  */
+LTTNG_HIDDEN
 int compat_epoll_set_max_size(void)
 {
 	int ret, fd, retval = 0;
@@ -261,7 +320,15 @@ int compat_epoll_set_max_size(void)
 
 	fd = open(COMPAT_EPOLL_PROC_PATH, O_RDONLY);
 	if (fd < 0) {
-		retval = -1;
+		/*
+		 * Failing on opening [1] is not an error per see. [1] was
+		 * introduced in Linux 2.6.28 but epoll is available since
+		 * 2.5.44. Hence, goto end and set a default value without
+		 * setting an error return value.
+		 *
+		 * [1] /proc/sys/fs/epoll/max_user_watches
+		 */
+		retval = 0;
 		goto end;
 	}
 
