@@ -2,6 +2,7 @@
  * Copyright (C) 2011 - Julien Desfossez <julien.desfossez@polymtl.ca>
  *                      Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
  *               2012 - David Goulet <dgoulet@efficios.com>
+ *               2018 - Jérémie Galarneau <jeremie.galarneau@efficios.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2 only,
@@ -33,6 +34,11 @@
 #include <common/sessiond-comm/sessiond-comm.h>
 #include <common/pipe.h>
 #include <common/index/ctf-index.h>
+#include <common/trace-chunk-registry.h>
+#include <common/credentials.h>
+#include <common/buffer-view.h>
+
+struct lttng_consumer_local_data;
 
 /* Commands for consumer */
 enum lttng_consumer_command {
@@ -61,13 +67,12 @@ enum lttng_consumer_command {
 	LTTNG_CONSUMER_DISCARDED_EVENTS,
 	LTTNG_CONSUMER_LOST_PACKETS,
 	LTTNG_CONSUMER_CLEAR_QUIESCENT_CHANNEL,
-};
-
-/* State of each fd in consumer */
-enum lttng_consumer_stream_state {
-	LTTNG_CONSUMER_ACTIVE_STREAM,
-	LTTNG_CONSUMER_PAUSE_STREAM,
-	LTTNG_CONSUMER_DELETE_STREAM,
+	LTTNG_CONSUMER_SET_CHANNEL_MONITOR_PIPE,
+	LTTNG_CONSUMER_ROTATE_CHANNEL,
+	LTTNG_CONSUMER_INIT,
+	LTTNG_CONSUMER_CREATE_TRACE_CHUNK,
+	LTTNG_CONSUMER_CLOSE_TRACE_CHUNK,
+	LTTNG_CONSUMER_TRACE_CHUNK_EXISTS,
 };
 
 enum lttng_consumer_type {
@@ -92,6 +97,12 @@ enum consumer_channel_type {
 	CONSUMER_CHANNEL_TYPE_DATA	= 1,
 };
 
+enum sync_metadata_status {
+	SYNC_METADATA_STATUS_NEW_DATA,
+	SYNC_METADATA_STATUS_NO_DATA,
+	SYNC_METADATA_STATUS_ERROR,
+};
+
 extern struct lttng_consumer_global_data consumer_data;
 
 struct stream_list {
@@ -103,26 +114,45 @@ struct stream_list {
 struct consumer_metadata_cache;
 
 struct lttng_consumer_channel {
+	/* Is the channel published in the channel hash tables? */
+	bool is_published;
+	/*
+	 * Was the channel deleted (logically) and waiting to be reclaimed?
+	 * If this flag is set, no modification that is not cleaned-up by the
+	 * RCU reclamation callback should be made
+	 */
+	bool is_deleted;
 	/* HT node used for consumer_data.channel_ht */
 	struct lttng_ht_node_u64 node;
+	/* HT node used for consumer_data.channels_by_session_id_ht */
+	struct lttng_ht_node_u64 channels_by_session_id_ht_node;
 	/* Indexed key. Incremented value in the consumer. */
 	uint64_t key;
 	/* Number of streams referencing this channel */
 	int refcount;
 	/* Tracing session id on the session daemon side. */
 	uint64_t session_id;
+	/* Current trace chunk of the session in which this channel exists. */
+	struct lttng_trace_chunk *trace_chunk;
 	/*
 	 * Session id when requesting metadata to the session daemon for
 	 * a session with per-PID buffers.
 	 */
 	uint64_t session_id_per_pid;
-	/* Channel trace file path name. */
+	/*
+	 * In the case of local streams, this field contains the channel's
+	 * output path; a path relative to the session's output path.
+	 *   e.g. ust/uid/1000/64-bit
+	 *
+	 * In the case of remote streams, the contents of this field depends
+	 * on the version of the relay daemon peer. For 2.11+ peers, the
+	 * contents are the same as in the local case. However, for legacy
+	 * peers, this contains a path of the form:
+	 *   /hostname/session_path/ust/uid/1000/64-bit
+	 */
 	char pathname[PATH_MAX];
 	/* Channel name. */
 	char name[LTTNG_SYMBOL_NAME_LEN];
-	/* UID and GID of the session owning this channel. */
-	uid_t uid;
-	gid_t gid;
 	/* Relayd id of the channel. -1ULL if it does not apply. */
 	uint64_t relayd_id;
 	/*
@@ -160,6 +190,7 @@ struct lttng_consumer_channel {
 
 	/* Metadata cache is metadata channel */
 	struct consumer_metadata_cache *metadata_cache;
+
 	/* For UST metadata periodical flush */
 	int switch_timer_enabled;
 	timer_t switch_timer;
@@ -169,6 +200,12 @@ struct lttng_consumer_channel {
 	int live_timer_enabled;
 	timer_t live_timer;
 	int live_timer_error;
+	/* Channel is part of a live session ? */
+	bool is_live;
+
+	/* For channel monitoring timer. */
+	int monitor_timer_enabled;
+	timer_t monitor_timer;
 
 	/* On-disk circular buffer */
 	uint64_t tracefile_size;
@@ -214,6 +251,8 @@ struct lttng_consumer_channel {
 	int nr_stream_fds;
 	char root_shm_path[PATH_MAX];
 	char shm_path[PATH_MAX];
+	/* Only set for UST channels. */
+	LTTNG_OPTIONAL(struct lttng_credentials) buffer_credentials;
 	/* Total number of discarded events for that channel. */
 	uint64_t discarded_events;
 	/* Total number of missed packets due to overwriting (overwrite). */
@@ -222,6 +261,150 @@ struct lttng_consumer_channel {
 	bool streams_sent_to_relayd;
 };
 
+struct stream_subbuffer {
+	union {
+		/*
+		 * CONSUMER_CHANNEL_SPLICE
+		 * No ownership assumed.
+		 */
+		int fd;
+		/* CONSUMER_CHANNEL_MMAP */
+		struct lttng_buffer_view buffer;
+	} buffer;
+	union {
+		/*
+		 * Common members are fine to access through either
+		 * union entries (as per C11, Common Initial Sequence).
+		 */
+		struct {
+			unsigned long subbuf_size;
+			unsigned long padded_subbuf_size;
+			uint64_t version;
+			/*
+			 * Left unset when unsupported.
+			 *
+			 * Indicates that this is the last sub-buffer of
+			 * a series of sub-buffer that makes-up a coherent
+			 * (parseable) unit of metadata.
+			 */
+			LTTNG_OPTIONAL(bool) coherent;
+		} metadata;
+		struct {
+			unsigned long subbuf_size;
+			unsigned long padded_subbuf_size;
+			uint64_t packet_size;
+			uint64_t content_size;
+			uint64_t timestamp_begin;
+			uint64_t timestamp_end;
+			uint64_t events_discarded;
+			/* Left unset when unsupported. */
+			LTTNG_OPTIONAL(uint64_t) sequence_number;
+			uint64_t stream_id;
+			/* Left unset when unsupported. */
+			LTTNG_OPTIONAL(uint64_t) stream_instance_id;
+		} data;
+	} info;
+};
+
+/*
+ * Perform any operation required to acknowledge
+ * the wake-up of a consumer stream (e.g. consume a byte on a wake-up pipe).
+ *
+ * Stream and channel locks are acquired during this call.
+ */
+typedef int (*on_wake_up_cb)(struct lttng_consumer_stream *);
+
+/*
+ * Perform any operation required before a consumer stream is put
+ * to sleep before awaiting a data availability notification.
+ *
+ * Stream and channel locks are acquired during this call.
+ */
+typedef int (*on_sleep_cb)(struct lttng_consumer_stream *,
+		struct lttng_consumer_local_data *);
+
+/*
+ * Acquire the subbuffer at the current 'consumed' position.
+ *
+ * Stream and channel locks are acquired during this call.
+ */
+typedef int (*get_next_subbuffer_cb)(struct lttng_consumer_stream *,
+		struct stream_subbuffer *);
+
+/*
+ * Populate the stream_subbuffer's info member. The info to populate
+ * depends on the type (metadata/data) of the stream.
+ *
+ * Stream and channel locks are acquired during this call.
+ */
+typedef int (*extract_subbuffer_info_cb)(
+		struct lttng_consumer_stream *, struct stream_subbuffer *);
+
+/*
+ * Invoked after a subbuffer's info has been filled.
+ *
+ * Stream and channel locks are acquired during this call.
+ */
+typedef int (*pre_consume_subbuffer_cb)(struct lttng_consumer_stream *,
+		const struct stream_subbuffer *);
+
+/*
+ * Consume subbuffer contents.
+ *
+ * Stream and channel locks are acquired during this call.
+ */
+typedef ssize_t (*consume_subbuffer_cb)(struct lttng_consumer_local_data *,
+		struct lttng_consumer_stream *,
+		const struct stream_subbuffer *);
+
+/*
+ * Release the current subbuffer and advance the 'consumed' position by
+ * one subbuffer.
+ *
+ * Stream and channel locks are acquired during this call.
+ */
+typedef int (*put_next_subbuffer_cb)(struct lttng_consumer_stream *,
+		struct stream_subbuffer *);
+
+/*
+ * Invoked after consuming a subbuffer.
+ *
+ * Stream and channel locks are acquired during this call.
+ */
+typedef int (*post_consume_cb)(struct lttng_consumer_stream *,
+		const struct stream_subbuffer *,
+		struct lttng_consumer_local_data *);
+
+/*
+ * Send a live beacon if no data is available.
+ *
+ * Stream and channel locks are acquired during this call.
+ */
+typedef int (*send_live_beacon_cb)(struct lttng_consumer_stream *);
+
+/*
+ * Lock the stream and channel locks and any other stream-type specific
+ * lock that need to be acquired during the processing of an
+ * availability notification.
+ */
+typedef void (*lock_cb)(struct lttng_consumer_stream *);
+
+/*
+ * Unlock the stream and channel locks and any other stream-type specific
+ * lock before sleeping until the next availability notification.
+ *
+ * Stream and channel locks are acquired during this call.
+ */
+typedef void (*unlock_cb)(struct lttng_consumer_stream *);
+
+/*
+ * Invoked when a subbuffer's metadata version does not match the last
+ * known metadata version.
+ *
+ * Stream and channel locks are acquired during this call.
+ */
+typedef void (*reset_metadata_cb)(struct lttng_consumer_stream *);
+
 /*
  * Internal representation of the streams, sessiond_key is used to identify
  * uniquely a stream.
@@ -235,6 +418,12 @@ struct lttng_consumer_stream {
 	struct lttng_ht_node_u64 node_session_id;
 	/* Pointer to associated channel. */
 	struct lttng_consumer_channel *chan;
+	/*
+	 * Current trace chunk. Holds a reference to the trace chunk.
+	 * `chunk` can be NULL when a stream is not associated to a chunk, e.g.
+	 * when it was created in the context of a no-output session.
+	 */
+	struct lttng_trace_chunk *trace_chunk;
 
 	/* Key by which the stream is indexed for 'node'. */
 	uint64_t key;
@@ -247,7 +436,6 @@ struct lttng_consumer_stream {
 	off_t out_fd_offset;
 	/* Amount of bytes written to the output */
 	uint64_t output_written;
-	enum lttng_consumer_stream_state state;
 	int shm_fd_is_copy;
 	int data_read;
 	int hangup_flush_done;
@@ -271,6 +459,11 @@ struct lttng_consumer_stream {
 	 */
 	bool quiescent;
 
+	/*
+	 * True if the sequence number is not available (lttng-modules < 2.8).
+	 */
+	bool sequence_number_unavailable;
+
 	/*
 	 * metadata_timer_lock protects flags waiting_on_metadata and
 	 * missed_metadata_flush.
@@ -286,7 +479,7 @@ struct lttng_consumer_stream {
 	bool missed_metadata_flush;
 
 	enum lttng_event_output output;
-	/* Maximum subbuffer size. */
+	/* Maximum subbuffer size (in bytes). */
 	unsigned long max_sb_size;
 
 	/*
@@ -299,9 +492,6 @@ struct lttng_consumer_stream {
 	/* For UST */
 
 	int wait_fd;
-	/* UID/GID of the user owning the session to which stream belongs */
-	uid_t uid;
-	gid_t gid;
 	/* Network sequence number. Indicating on which relayd socket it goes. */
 	uint64_t net_seq_idx;
 	/*
@@ -407,6 +597,30 @@ struct lttng_consumer_stream {
 	pthread_cond_t metadata_rdv;
 	pthread_mutex_t metadata_rdv_lock;
 
+	/*
+	 * rotate_position represents the packet sequence number of the last
+	 * packet which belongs to the current trace chunk prior to the rotation.
+	 * When that position is reached, this tracefile can be closed and a
+	 * new one is created in channel_read_only_attributes.path.
+	 */
+	uint64_t rotate_position;
+
+	/*
+	 * Read-only copies of channel values. We cannot safely access the
+	 * channel from a stream, so we need to have a local copy of these
+	 * fields in the stream object. These fields should be removed from
+	 * the stream objects when we introduce refcounting.
+	 */
+	struct {
+		uint64_t tracefile_size;
+	} channel_read_only_attributes;
+
+	/*
+	 * Flag to inform the data or metadata thread that a stream is
+	 * ready to be rotated.
+	 */
+	bool rotate_ready;
+
 	/* Indicate if the stream still has some data to be read. */
 	unsigned int has_data:1;
 	/*
@@ -414,6 +628,25 @@ struct lttng_consumer_stream {
 	 * file before writing in it (regeneration).
 	 */
 	unsigned int reset_metadata_flag:1;
+	struct {
+		/*
+		 * Invoked in the order of declaration.
+		 * See callback type definitions.
+		 */
+		lock_cb lock;
+		on_wake_up_cb on_wake_up;
+		get_next_subbuffer_cb get_next_subbuffer;
+		extract_subbuffer_info_cb extract_subbuffer_info;
+		pre_consume_subbuffer_cb pre_consume_subbuffer;
+		reset_metadata_cb reset_metadata;
+		consume_subbuffer_cb consume_subbuffer;
+		put_next_subbuffer_cb put_next_subbuffer;
+		post_consume_cb post_consume;
+		send_live_beacon_cb send_live_beacon;
+		on_sleep_cb on_sleep;
+		unlock_cb unlock;
+	} read_subbuffer_ops;
+	struct metadata_bucket *metadata_bucket;
 };
 
 /*
@@ -470,7 +703,8 @@ struct lttng_consumer_local_data {
 	 * Returns the number of bytes read, or negative error value.
 	 */
 	ssize_t (*on_buffer_ready)(struct lttng_consumer_stream *stream,
-			struct lttng_consumer_local_data *ctx);
+			struct lttng_consumer_local_data *ctx,
+			bool locked_by_caller);
 	/*
 	 * function to call when we receive a new channel, it receives a
 	 * newly allocated channel, depending on the return code of this
@@ -542,6 +776,12 @@ struct lttng_consumer_local_data {
 	int consumer_should_quit[2];
 	/* Metadata poll thread pipe. Transfer metadata stream to it */
 	struct lttng_pipe *consumer_metadata_pipe;
+	/*
+	 * Pipe used by the channel monitoring timers to provide state samples
+	 * to the session daemon (write-only).
+	 */
+	int channel_monitor_pipe;
+	LTTNG_OPTIONAL(lttng_uuid) sessiond_uuid;
 };
 
 /*
@@ -566,6 +806,8 @@ struct lttng_consumer_global_data {
 
 	/* Channel hash table protected by consumer_data.lock. */
 	struct lttng_ht *channel_ht;
+	/* Channel hash table indexed by session id. */
+	struct lttng_ht *channels_by_session_id_ht;
 	/*
 	 * Flag specifying if the local array of FDs needs update in the
 	 * poll function. Protected by consumer_data.lock.
@@ -592,13 +834,45 @@ struct lttng_consumer_global_data {
 	 * This HT uses the "node_channel_id" of the consumer stream.
 	 */
 	struct lttng_ht *stream_per_chan_id_ht;
+
+	/*
+	 * Trace chunk registry indexed by (session_id, chunk_id).
+	 */
+	struct lttng_trace_chunk_registry *chunk_registry;
 };
 
 /*
  * Set to nonzero when the consumer is exiting. Updated by signal
  * handler and thread exit, read by threads.
  */
-extern volatile int consumer_quit;
+extern int consumer_quit;
+
+/*
+ * Set to nonzero when the consumer is exiting. Updated by signal
+ * handler and thread exit, read by threads.
+ */
+extern int consumer_quit;
+
+/* Flag used to temporarily pause data consumption from testpoints. */
+extern int data_consumption_paused;
+
+/* Return a human-readable consumer type string that is suitable for logging. */
+static inline
+const char *lttng_consumer_type_str(enum lttng_consumer_type type)
+{
+	switch (type) {
+	case LTTNG_CONSUMER_UNKNOWN:
+		return "unknown";
+	case LTTNG_CONSUMER_KERNEL:
+		return "kernel";
+	case LTTNG_CONSUMER32_UST:
+		return "32-bit user space";
+	case LTTNG_CONSUMER64_UST:
+		return "64-bit user space";
+	default:
+		abort();
+	}
+}
 
 /*
  * Init consumer data structures.
@@ -641,24 +915,31 @@ void lttng_consumer_cleanup(void);
  */
 int lttng_consumer_poll_socket(struct pollfd *kconsumer_sockpoll);
 
-struct lttng_consumer_stream *consumer_allocate_stream(uint64_t channel_key,
+/*
+ * Copy the fields from the channel that need to be accessed (read-only)
+ * directly from the stream.
+ */
+void consumer_stream_update_channel_attributes(
+		struct lttng_consumer_stream *stream,
+		struct lttng_consumer_channel *channel);
+
+struct lttng_consumer_stream *consumer_allocate_stream(
+		struct lttng_consumer_channel *channel,
+		uint64_t channel_key,
 		uint64_t stream_key,
-		enum lttng_consumer_stream_state state,
 		const char *channel_name,
-		uid_t uid,
-		gid_t gid,
 		uint64_t relayd_id,
 		uint64_t session_id,
+		struct lttng_trace_chunk *trace_chunk,
 		int cpu,
 		int *alloc_ret,
 		enum consumer_channel_type type,
 		unsigned int monitor);
 struct lttng_consumer_channel *consumer_allocate_channel(uint64_t key,
 		uint64_t session_id,
+		const uint64_t *chunk_id,
 		const char *pathname,
 		const char *name,
-		uid_t uid,
-		gid_t gid,
 		uint64_t relayd_id,
 		enum lttng_event_output output,
 		uint64_t tracefile_size,
@@ -666,6 +947,7 @@ struct lttng_consumer_channel *consumer_allocate_channel(uint64_t key,
 		uint64_t session_id_per_pid,
 		unsigned int monitor,
 		unsigned int live_timer_interval,
+		bool is_in_live_session,
 		const char *root_shm_path,
 		const char *shm_path);
 void consumer_del_stream(struct lttng_consumer_stream *stream,
@@ -689,24 +971,26 @@ void consumer_steal_stream_key(int key, struct lttng_ht *ht);
 struct lttng_consumer_local_data *lttng_consumer_create(
 		enum lttng_consumer_type type,
 		ssize_t (*buffer_ready)(struct lttng_consumer_stream *stream,
-			struct lttng_consumer_local_data *ctx),
+			struct lttng_consumer_local_data *ctx,
+			bool locked_by_caller),
 		int (*recv_channel)(struct lttng_consumer_channel *channel),
 		int (*recv_stream)(struct lttng_consumer_stream *stream),
 		int (*update_stream)(uint64_t sessiond_key, uint32_t state));
 void lttng_consumer_destroy(struct lttng_consumer_local_data *ctx);
 ssize_t lttng_consumer_on_read_subbuffer_mmap(
-		struct lttng_consumer_local_data *ctx,
-		struct lttng_consumer_stream *stream, unsigned long len,
-		unsigned long padding,
-		struct ctf_packet_index *index);
+		struct lttng_consumer_stream *stream,
+		const struct lttng_buffer_view *buffer,
+		unsigned long padding);
 ssize_t lttng_consumer_on_read_subbuffer_splice(
 		struct lttng_consumer_local_data *ctx,
 		struct lttng_consumer_stream *stream, unsigned long len,
-		unsigned long padding,
-		struct ctf_packet_index *index);
+		unsigned long padding);
+int lttng_consumer_sample_snapshot_positions(struct lttng_consumer_stream *stream);
 int lttng_consumer_take_snapshot(struct lttng_consumer_stream *stream);
 int lttng_consumer_get_produced_snapshot(struct lttng_consumer_stream *stream,
 		unsigned long *pos);
+int lttng_consumer_get_consumed_snapshot(struct lttng_consumer_stream *stream,
+		unsigned long *pos);
 int lttng_ustconsumer_get_wakeup_fd(struct lttng_consumer_stream *stream);
 int lttng_ustconsumer_close_wakeup_fd(struct lttng_consumer_stream *stream);
 void *consumer_thread_metadata_poll(void *data);
@@ -717,9 +1001,10 @@ int lttng_consumer_recv_cmd(struct lttng_consumer_local_data *ctx,
 		int sock, struct pollfd *consumer_sockpoll);
 
 ssize_t lttng_consumer_read_subbuffer(struct lttng_consumer_stream *stream,
-		struct lttng_consumer_local_data *ctx);
+		struct lttng_consumer_local_data *ctx,
+		bool locked_by_caller);
 int lttng_consumer_on_recv_stream(struct lttng_consumer_stream *stream);
-int consumer_add_relayd_socket(uint64_t net_seq_idx, int sock_type,
+void consumer_add_relayd_socket(uint64_t net_seq_idx, int sock_type,
 		struct lttng_consumer_local_data *ctx, int sock,
 		struct pollfd *consumer_sockpoll, struct lttcomm_relayd_sock *relayd_sock,
 		uint64_t sessiond_id, uint64_t relayd_session_id);
@@ -735,11 +1020,38 @@ void consumer_destroy_relayd(struct consumer_relayd_sock_pair *relayd);
 unsigned long consumer_get_consume_start_pos(unsigned long consumed_pos,
 		unsigned long produced_pos, uint64_t nb_packets_per_stream,
 		uint64_t max_sb_size);
-int consumer_add_data_stream(struct lttng_consumer_stream *stream);
+void consumer_add_data_stream(struct lttng_consumer_stream *stream);
 void consumer_del_stream_for_data(struct lttng_consumer_stream *stream);
-int consumer_add_metadata_stream(struct lttng_consumer_stream *stream);
+void consumer_add_metadata_stream(struct lttng_consumer_stream *stream);
 void consumer_del_stream_for_metadata(struct lttng_consumer_stream *stream);
 int consumer_create_index_file(struct lttng_consumer_stream *stream);
+int lttng_consumer_rotate_channel(struct lttng_consumer_channel *channel,
+		uint64_t key, uint64_t relayd_id, uint32_t metadata,
+		struct lttng_consumer_local_data *ctx);
+int lttng_consumer_stream_is_rotate_ready(struct lttng_consumer_stream *stream);
+int lttng_consumer_rotate_stream(struct lttng_consumer_local_data *ctx,
+		struct lttng_consumer_stream *stream);
+int lttng_consumer_rotate_ready_streams(struct lttng_consumer_channel *channel,
+		uint64_t key, struct lttng_consumer_local_data *ctx);
+void lttng_consumer_reset_stream_rotate_state(struct lttng_consumer_stream *stream);
+enum lttcomm_return_code lttng_consumer_create_trace_chunk(
+		const uint64_t *relayd_id, uint64_t session_id,
+		uint64_t chunk_id,
+		time_t chunk_creation_timestamp,
+		const char *chunk_override_name,
+		const struct lttng_credentials *credentials,
+		struct lttng_directory_handle *chunk_directory_handle);
+enum lttcomm_return_code lttng_consumer_close_trace_chunk(
+		const uint64_t *relayd_id, uint64_t session_id,
+		uint64_t chunk_id, time_t chunk_close_timestamp,
+		const enum lttng_trace_chunk_command_type *close_command,
+		char *path);
+enum lttcomm_return_code lttng_consumer_trace_chunk_exists(
+		const uint64_t *relayd_id, uint64_t session_id,
+		uint64_t chunk_id);
 void lttng_consumer_cleanup_relayd(struct consumer_relayd_sock_pair *relayd);
+enum lttcomm_return_code lttng_consumer_init_command(
+		struct lttng_consumer_local_data *ctx,
+		const lttng_uuid sessiond_uuid);
 
 #endif /* LIB_CONSUMER_H */
