@@ -39,9 +39,6 @@ int lttng_opt_quiet = 1;
 int lttng_opt_verbose;
 int lttng_opt_mi;
 
-int ust_consumerd32_fd;
-int ust_consumerd64_fd;
-
 static const char alphanum[] =
 	"0123456789"
 	"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
@@ -95,17 +92,17 @@ static void test_create_kernel_metadata(void)
 	ok(kern->metadata->fd == -1 &&
 	   kern->metadata->conf != NULL &&
 	   kern->metadata->conf->attr.overwrite
-			== DEFAULT_CHANNEL_OVERWRITE &&
+			== DEFAULT_METADATA_OVERWRITE &&
 	   kern->metadata->conf->attr.subbuf_size
 			== default_get_metadata_subbuf_size() &&
 	   kern->metadata->conf->attr.num_subbuf
 			== DEFAULT_METADATA_SUBBUF_NUM &&
 	   kern->metadata->conf->attr.switch_timer_interval
-			== DEFAULT_KERNEL_CHANNEL_SWITCH_TIMER &&
+			== DEFAULT_METADATA_SWITCH_TIMER &&
 	   kern->metadata->conf->attr.read_timer_interval
-			== DEFAULT_KERNEL_CHANNEL_READ_TIMER &&
+			== DEFAULT_METADATA_READ_TIMER &&
 	   kern->metadata->conf->attr.output
-	       == DEFAULT_KERNEL_CHANNEL_OUTPUT,
+			== LTTNG_EVENT_MMAP,
 	   "Validate kernel session metadata");
 
 	trace_kernel_destroy_metadata(kern->metadata);
@@ -115,8 +112,11 @@ static void test_create_kernel_channel(void)
 {
 	struct ltt_kernel_channel *chan;
 	struct lttng_channel attr;
+	struct lttng_channel_extended extended;
 
 	memset(&attr, 0, sizeof(attr));
+	memset(&extended, 0, sizeof(extended));
+	attr.attr.extended.ptr = &extended;
 
 	chan = trace_kernel_create_channel(&attr);
 	ok(chan != NULL, "Create kernel channel");
@@ -139,18 +139,19 @@ static void test_create_kernel_channel(void)
 
 static void test_create_kernel_event(void)
 {
+	enum lttng_error_code ret;
 	struct ltt_kernel_event *event;
 	struct lttng_event ev;
 
 	memset(&ev, 0, sizeof(ev));
 	ok(!lttng_strncpy(ev.name, get_random_string(),
-			LTTNG_KERNEL_SYM_NAME_LEN),
+			RANDOM_STRING_LEN),
 		"Validate string length");
 	ev.type = LTTNG_EVENT_TRACEPOINT;
 	ev.loglevel_type = LTTNG_EVENT_LOGLEVEL_ALL;
 
-	event = trace_kernel_create_event(&ev, NULL, NULL);
-	ok(event != NULL, "Create kernel event");
+	ret = trace_kernel_create_event(&ev, NULL, NULL, &event);
+	ok(ret == LTTNG_OK, "Create kernel event");
 
 	if (!event) {
 		skip(1, "Event is null");
@@ -201,6 +202,5 @@ int main(int argc, char **argv)
 	test_create_kernel_event();
 	test_create_kernel_stream();
 
-	/* Success */
-	return 0;
+	return exit_status();
 }
